%% Learning the Whole Skeleton of a Bayesian Network
% This demonstration illustrates the skeleton learning process of the
% example Weather network.
%% Loading the True Network
% First we load the ALARM network in the |org.mensxmachina.bnet.Network|
% object |net|.

% load ALARM network
load mxm_bnet_alarm_net; % loads 'net' Network

%% Loading the Network Samples
% Then we load samples from the ALARM network in dataset |a|.

% load ALARM network samples
load mxm_bnet_alarm_samples; % loads 'a' dataset

%% Learning the Skeleton from the Samples
% We provide the samples as input to the |org.mensxmachina.bnet.mmskeleton|
% function, which applies the MMPC-bar algorithm to each of the variables
% in the dataset. We don't supply any parameter-value pairs so the defaults
% are used.

% learn the skeleton
skeleton = org.mensxmachina.bnet.mmskeleton(a);

%% Comparing the Learned Skeleton with the True One
% Now we compare the learned skeleton with the true Weather skeleton. First
% we obtain the skeleton of the Weather network by calling the |skeleton|
% method of the |org.mensxmachina.bnet.Network| class. Then we get a
% classification performance object with
% |org.mensxmachina.bnet.skeletonperf| and we print the sensitivity and 
% specificity. |org.mensxmachina.bnet.skeletonperf| is a version of the 
% |classperf| function in the Bioinformatics Toolbox (TM), specialized for 
% skeleton learning. A skeleton learning task can be viewed as a binary
% classification task where the possible edges are classified as edges 
% (positives) or non-edges (negatives). Finally we plot the confusion
% matrix with the |org.mensxmachina.bnet.plotskeletonconfusion| function,
% which is a version of the |plotconfusion| function in the Neural Network 
% Toolbox (TM), specialized for skeleton learning. 

% get true skeleton
skeleton_true = net.skeleton;

% get classifier performance object
cp = org.mensxmachina.bnet.skeletonperf(skeleton_true, skeleton);

% print sensitivity and specificity
fprintf('\nSensitivity = %.2f%%\n', cp.sensitivity*100);
fprintf('\nSpecificity = %.2f%%\n', cp.specificity*100);

% plot confusion matrix
org.mensxmachina.bnet.plotskeletonconfusion(skeleton_true, skeleton);