function plotpcconfusion(targetskeleton, outputpc, t)
%PLOTPCCONFUSION Plot parents and children confusion matrix
%
%   ORG.MENSXMACHINA.BNET.PLOTPCCONFUSION is a version of the PLOTCONFUSION
%   function in the Neural Network Toolbox (TM), specialized for parents
%   and children learning. For a given node, the task of learning its 
%   parents and children can be viewed as a binary classification task,
%   where each other node is classified as either parent or child, or not.
%   
%   PLOTSKELETONCONFUSION(TARGETSKELETON, OUTPUTPC, T) displays the 
%   classification confusion grid for the output parents and children set
%   OUTPUTPC, returned either by ORG.MENSXMACHINA.BNET.MMPC or
%   ORG.MENSXMACHINA.BNET.MMPCBAR, targeting the parents and children of
%   the variable specified by positive integer T in skeleton
%   TARGETSKELETON.
%
%   See also: PLOTCONFUSION

%   Copyright 2010 Mens X Machina
%
%   This file is part of Mens X Machina Bayesian Network Toolbox.
%
%   Mens X Machina Bayesian Network Toolbox is free software: you can
%   redistribute it and/or modify it under the terms of the GNU General
%   Public License as published by the Free Software Foundation, either 
%   version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Bayesian Network Toolbox is distributed in the hope that
%   it will be useful, but WITHOUT ANY WARRANTY; without even the implied
%   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
%   the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with Mens X Machina Bayesian Network Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

targets = targetskeleton(t, :);
targets(t) = []; % remove target variable

if islogical(outputpc)
    outputs = outputpc;  
else   
    % convert to logical indices
    outputs = false(1, size(targetskeleton, 1));
    outputs(outputpc) = true;    
end

outputs(t) = []; % remove target variable

plotconfusion(targets, outputs); 

end