function [tpr fpr threshold] = plotpcroc(targetskeleton, pMax, t)
%PLOTPCROC Plot parents and children learning ROC
%
%   PLOTPCROC is called just like ORG.MENSXMACHINA.BNET.PCROC, but it also
%   plots the Receiver Operating Characteristic.
%
%   Example:
%
%       % load ALARM network
%       load mxm_bnet_alarm_net; % loads 'net' Network
%
%       % load ALARM network samples
%       load mxm_bnet_alarm_samples; % loads 'a' dataset
%
%       % learn parents and children set of variable #37
%       [pc pMax pMaxStat] = ...
%           org.mensxmachina.bnet.mmpc(a, 37, 'includeNpc', true);
%
%       % get true skeleton
%       skeleton_true = net.skeleton;
%
%       % plot ROC for variable #37
%       [tpr fpr threshold] = ..
%           org.mensxmachina.bnet.plotpcroc(skeleton_true, pMax, 37);
%
%   See also: 
%       ORG.MENSXMACHINA.COMMONS.STATS.ROC, ORG.MENSXMACHINA.BNET.PCROC,
%       ORG.MENSXMACHINA.BNET.PCAUC 

%   Copyright 2010 Mens X Machina
%
%   This file is part of Mens X Machina Bayesian Network Toolbox.
%
%   Mens X Machina Bayesian Network Toolbox is free software: you can
%   redistribute it and/or modify it under the terms of the GNU General
%   Public License as published by the Free Software Foundation, either 
%   version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Bayesian Network Toolbox is distributed in the hope that
%   it will be useful, but WITHOUT ANY WARRANTY; without even the implied
%   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
%   the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with Mens X Machina Bayesian Network Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

if ~iscell(targetskeleton)

    % get roc inputs
    [targets outputs] = rocinputs(targetskeleton, pMax, t);

else
        
    targets = cell(size(targetskeleton));
    outputs = cell(size(targetskeleton));
    
    for i=1:length(targetskeleton)
    
        % get roc inputs
        [targets{i} outputs{i}] = ...
            rocinputs(targetskeleton{i}, pMax{i}, t(i));
    
    end
    
end

% plot ROC
[tpr fpr threshold] = ...
    org.mensxmachina.commons.stats.plotroc(targets, outputs);

end

function [targets outputs] = rocinputs(targetskeleton, pMax, t)

targets = targetskeleton(t, :);
targets(t) = []; % remove target variable

outputs = -pMax;
outputs(t) = []; % remove target variable

end