function [tpr fpr threshold] = plotskeletonroc(targetskeleton, pMax)
%PLOTSKELETONROC Plot skeleton learner receiver operating characteristic
%
%   PLOTSKELETONROC is called just like ORG.MENSXMACHINA.BNET.SKELETONROC,
%   but it also plots the Receiver Operating Characteristic.
%
%   Example:
%
%       % load ALARM network samples
%       load mxm_bnet_alarm_samples; % loads 'a' dataset
%
%       % learn skeleton
%       [skeleton pMax pMaxStat] = ...
%           org.mensxmachina.bnet.mmskeleton(a, 'includeNonEdges', true);
%
%       % load ALARM network
%       load mxm_bnet_alarm_net; % loads 'net' network
%
%       % get true skeleton
%       skeleton_true = net.skeleton;
%
%       % plot ROC
%       [tpr fpr threshold] = ...
%           org.mensxmachina.bnet.plotskeletonroc(skeleton_true, pMax)
%
%   See also: 
%       ORG.MENSXMACHINA.BNET.SKELETONROC,
%       ORG.MENSXMACHINA.BNET.SKELETONAUC,
%       ORG.MENSXMACHINA.COMMONS.STATS.PLOTROC

%   Copyright 2010 Mens X Machina
%
%   This file is part of Mens X Machina Bayesian Network Toolbox.
%
%   Mens X Machina Bayesian Network Toolbox is free software: you can
%   redistribute it and/or modify it under the terms of the GNU General
%   Public License as published by the Free Software Foundation, either 
%   version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Bayesian Network Toolbox is distributed in the hope that
%   it will be useful, but WITHOUT ANY WARRANTY; without even the implied
%   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
%   the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with Mens X Machina Bayesian Network Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

% convert skeletons to vectors
targets = org.mensxmachina.commons.graph.skeleton2vector(targetskeleton);

% convert p-value matrices to vectors
outputs = org.mensxmachina.commons.graph.skeleton2vector(pMax);

if ~iscell(pMax)
    outputs = -outputs;
else
    outputs = cellfun(@uminus, outputs, 'UniformOutput', false);
end

[tpr fpr threshold] = org.mensxmachina.commons.stats.plotroc(targets, outputs);

end