function net = weather
%WEATHER Creates the weather example bayesian network

%   Copyright 2010 Mens X Machina
%
%   This file is part of Mens X Machina Bayesian Network Toolbox.
%
%   Mens X Machina Bayesian Network Toolbox is free software: you can
%   redistribute it and/or modify it under the terms of the GNU General
%   Public License as published by the Free Software Foundation, either 
%   version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Bayesian Network Toolbox is distributed in the hope that
%   it will be useful, but WITHOUT ANY WARRANTY; without even the implied
%   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
%   the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with Mens X Machina Bayesian Network Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

node = struct('Cloudy', 1, ...
              'Sprinkler', 2, ...
              'Rain', 3, ...
              'WetGrass', 4);

G = sparse(4, 4);
G([node.Cloudy], node.Sprinkler) = 1;
G([node.Cloudy], node.Rain) = 1;
G([node.Rain node.Sprinkler], node.WetGrass) = 1;

Labels = {{'false', 'true'}, ...
         {'false', 'true'}, ...
         {'false', 'true'}, ...
         {'false', 'true'}};

P{node.Cloudy} = [0.5; 0.5];
P{node.Sprinkler} = reshape([0.5 0.5 0.9 0.1], [2 2]);
P{node.Rain} = reshape([0.8 0.2 0.2 0.8], [2 2]);
P{node.WetGrass} = reshape([0.01 0.99 1.0 0.0 0.1 0.9 0.1 0.9], [2 2 2]);

description = ['Example from Russell and Norvig, "Artificial' ...
               'Intelligence: a Modern Approach", Prentice Hall, ' ...
               '1995, page 454.'];

VarNames = fieldnames(node)';

net = org.mensxmachina.bnet.Network(G, P, 'VarNames', VarNames, 'Labels', Labels, 'ClassNames', 'nominal', 'Description', description);

end
