classdef TestMmSkeleton < TestCase
%TESTMMSKELETON mmskeleton test cases

properties
    
    net % net info
    
    netParams % net parameters
    sampleParams % sampling parameters
    learnParams % learning parameters
    
    config_default
    
end

methods

function obj = TestMmSkeleton(name)

obj = obj@TestCase(name);

%netsave();
%netssave();
%netparamssave();
%sampleparamssave();
%samplessave();

% load net info
load net;
obj.net = net;

% load net parameters
load netparams;
obj.netParams = netParams;
obj.netParams{'netName', 'values'} = obj.netParams{'netName', 'values'}(1:2);

% load sample parameters
load sampleparams;
obj.sampleParams = sampleParams;

obj.learnParams = dataset( ...
    {{[0 1 2 3]'; [0.01 0.05 0.1]'; [ true; false ]; [ true; false ]; [ true; false ]}, 'values'}, ...
    {{1; 0.05; true; false; false}, 'default'}, ...
    'ObsNames', {'rl', 'alpha', 'sc', 'includeNonEdges', 'verbose'} ...
    );

% create default config
obj.config_default = dataset( ...
    {{obj.netParams{'netName', 'default'}}, 'netName'}, ...
    {obj.netParams{'nsmax', 'default'}, 'nsmax'}, ...
    {obj.sampleParams{'sampleNo', 'default'}, 'sampleNo'}, ...
    {obj.sampleParams{'sampleSize', 'default'}, 'sampleSize'}, ...
    {obj.learnParams{'rl', 'default'}, 'rl'}, ...
    {obj.learnParams{'alpha', 'default'}, 'alpha'}, ...
    {obj.learnParams{'sc', 'default'}, 'sc'}, ...
    {obj.learnParams{'includeNonEdges', 'default'}, 'includeNonEdges'}, ...
    {obj.learnParams{'verbose', 'default'}, 'verbose'} ...
    );

end

function testIncludeNonEdges(obj)

Config = repmat(obj.config_default, length(obj.learnParams{'includeNonEdges', 'values'}), 1);

Config.includeNonEdges = obj.learnParams{'includeNonEdges', 'values'};

Run = obj.configRun(Config);

save('mmskeleton_run_var_includenonedges', 'Run');

assertEqual( Run.skeleton{1}, Run.skeleton{2} );

isSkeleton = find(Run.skeleton{1});

assertTrue( issparse(Run.skeleton{1}) );
assertTrue( ~issparse(Run.pMax{1}) );
assertTrue( ~issparse(Run.pMaxStat{1}) );

assertTrue( issparse(Run.skeleton{2}) );
assertTrue( issparse(Run.pMax{2}) );
assertTrue( issparse(Run.pMaxStat{2}) );

assertEqual( Run.pMax{1}(isSkeleton), full(Run.pMax{2}(isSkeleton)) );
assertEqual( Run.pMaxStat{1}(isSkeleton), full(Run.pMaxStat{2}(isSkeleton)) );

end

function testVerbose(obj)

Config = repmat(obj.config_default, length(obj.learnParams{'verbose', 'values'}), 1);
Config.verbose = obj.learnParams{'verbose', 'values'};

Run = obj.configRun(Config);

assertEqual( Run.skeleton{1}, Run.skeleton{2} );
assertEqual( Run.pMax{1}, Run.pMax{2} );
assertEqual( Run.pMaxStat{1}, Run.pMaxStat{2} );

end

function testDefaultConfig(obj)
    
Run = obj.configRun(obj.config_default);
Run = [ obj.config_default Run ];
save('mmskeleton_default_run', 'Run');
obj.runAnalyse('default');

end

function testCustomConfig(obj)
    
Config = obj.config_default;
Config.netName = {'alarm'};
Config.nsmax = 10;
Config.sampleSize = 100;
Config.rl = 1;
Run = obj.configRun(Config);
Run = [ Config Run ];
save('mmskeleton_custom_run', 'Run');
obj.runAnalyse('custom');

end

function testDefaultConfigCmpWithCe(obj)
    
Run_mxm = obj.configRun(obj.config_default);
Run_mxm = [ obj.config_default Run_mxm ];
Run_mxm.implId = {'MxM'};

Run_ce = obj.ceConfigRun(obj.config_default);
Run_ce = [ obj.config_default Run_ce ];
Run_ce.implId = {'CE'};

Run = [ Run_mxm; Run_ce ];

save('mmskeleton_cmp_with_ce_default', 'Run');

end

function testCustomCmpWithCe(obj)
    
%diary('mmskeleton_custom_cmp_with_ce.txt');

% compare with whatever configuration:
Config = obj.config_default;
Config.netName = {'alarm'};
Config.nsmax = 10;
Config.sampleSize = 500;
Config.rl = 2;
Config.sc = false;
Config.verbose = false;

nConfig = size(Config, 1);

Run_mxm = obj.configRun(Config);
Run_mxm = [Config Run_mxm];
Run_mxm.implId = repmat({'MxM'}, nConfig, 1);

%reuse = true;
%if reuse && exist('mmskeleton_a_cmp_with_ce_ce.mat', 'file')
%    load('mmskeleton_a_cmp_with_ce_ce', 'Run');
%    Run_ce = Run;
%else

Run_ce = obj.ceConfigRun(Config);
Run_ce = [Config Run_ce];
Run_ce.implId = repmat({'CE'}, nConfig, 1);

%Run = Run_ce;   
%save('mmskeleton_a_cmp_with_ce_ce', 'Run');
%end

Run = [ Run_mxm; Run_ce ];

save('mmskeleton_custom_cmp_with_ce', 'Run');
%diary off;

obj.analyseCmpWithCe('custom');

end

function testVarSsRl(obj)  

nConfigs = size(obj.net, 1)*length(obj.sampleParams{'sampleNo', 'values'})*length(obj.sampleParams{'sampleSize', 'values'})*length(obj.learnParams{'rl', 'values'});

% replicate default config
Config = repmat(obj.config_default, nConfigs, 1);

n = 0;

for i=1:size(obj.net, 1) % for each net
    for j=1:numel(obj.sampleParams{'sampleNo', 'values'}) % for each sample no
        for k=1:numel(obj.sampleParams{'sampleSize', 'values'}) % for each sample size
            for l=1:numel(obj.learnParams{'rl', 'values'}) % for each reuse level
                
                n = n + 1;
                
                Config.netName{n} = obj.netParams{'netName', 'values'}{i};
                Config.nsmax(n) = obj.netParams{'nsmax', 'values'}(i);    
                Config.sampleNo(n) = obj.sampleParams{'sampleNo', 'values'}(j);                
                Config.sampleSize(n) = obj.sampleParams{'sampleSize', 'values'}(k);               
                Config.rl(n) = obj.learnParams{'rl', 'values'}(l);
                
            end
        end
    end
end

Run = obj.configRun(Config);

Run = [ Config Run ];

save('mmskeleton_run_var_ss_rl', 'Run');

obj.analyseVarSsRl();

end

function testVarSs(obj)
    
nConfigs = size(obj.net, 1)*numel(obj.sampleParams{'sampleNo', 'values'})*numel(obj.sampleParams{'sampleSize', 'values'});

% replicate default config for each net, sample no, sample size and reuse level
Config = repmat(obj.config_default, nConfigs, 1);

n = 0;

for i=1:size(obj.net, 1) % for each net 
    for j=1:numel(obj.sampleParams{'sampleNo', 'values'}) % for each sample no
        for k=1:numel(obj.sampleParams{'sampleSize', 'values'}) % for each sample size
                
            n = n + 1;
                
            Config.netName{n} = obj.netParams{'netName', 'values'}{i};
            Config.nsmax(n) = obj.netParams{'nsmax', 'values'}(i);            
            Config.sampleNo(n) = obj.sampleParams{'sampleNo', 'values'}(j);                
            Config.sampleSize(n) = obj.sampleParams{'sampleSize', 'values'}(k);
            Config.includeNonEdges(n) = true; % include non-edge data
            
        end
    end
end

% debug
%Config = Config(1:2, :);
%nConfigs = 2;

Run = obj.configRun(Config);
Run = [ Config Run ];

save('mmskeleton_var_ss', 'Run');

obj.analyseVarSs();

end

function testCmpWithCeVarSs(obj)
    
nConfigs = size(obj.net, 1)*numel(obj.sampleParams{'sampleNo', 'values'})*numel(obj.sampleParams{'sampleSize', 'values'});

% replicate default config for each net, sample no, sample size and reuse level
Config = repmat(obj.config_default, nConfigs, 1);

n = 0;

for i=1:size(obj.net, 1) % for each net
    for j=1:numel(obj.sampleParams{'sampleNo', 'values'}) % for each sample no
        for k=1:numel(obj.sampleParams{'sampleSize', 'values'}) % for each sample size
                
            n = n + 1;
                
            Config.netName{n} = obj.netParams{'netName', 'values'}{i};
            Config.nsmax(n) = obj.netParams{'nsmax', 'values'}(i);             
            Config.sampleNo(n) = obj.sampleParams{'sampleNo', 'values'}(j);                
            Config.sampleSize(n) = obj.sampleParams{'sampleSize', 'values'}(k);
            
        end
    end
end

% debug
%Config = Config(1:2, :);
%nConfigs = 2;

Run_mxm = obj.configRun(Config);
Run_mxm = [ Config Run_mxm ];
Run_mxm.implId = repmat({'MxM'}, nConfigs, 1);

Run_ce = obj.ceConfigRun(Config);
Run_ce = [ Config Run_ce ];
Run_ce.implId = repmat({'CE'}, nConfigs, 1);

Run = [ Run_mxm; Run_ce ];

save('mmskeleton_cmp_with_ce_var_ss', 'Run');

obj.analyseCmpWithCeVarSs();

end

end

methods(Static)

function Run = configRun(Config)
   
Config    
    
nConfigs = size(Config, 1);

% create runs dataset
time = zeros(nConfigs, 1);
skeleton = cell(nConfigs, 1);
pMax = cell(nConfigs, 1);
pMaxStat = cell(nConfigs, 1);

Run = dataset(time, skeleton, pMax, pMaxStat);

for i=1:nConfigs % for each configuration
    
    fprintf('\nRunning configuration %d of %d (%.2f%%)\n', i, nConfigs, i/nConfigs*100);

    a = [];
    
    % load sample
    load(sprintf('%s_sample_%d', Config.netName{i}, Config.sampleNo(i)), 'a');
    
    % precompute subscripts
    subs = double(a);

    % precompute number of levels
    nLevels = org.mensxmachina.commons.stats.nlevels(a);

    disp('Learning skeleton with mmskeleton...');

    % dataset of this sample size
    data_ss = a(1:Config.sampleSize(i), :);

    % subscripts of this sample size
    subs_ss = subs(1:Config.sampleSize(i), :);

    mmskeleton_params = { ...
        'alpha', Config.alpha(i), ...
        'nsmax', Config.nsmax(i), ...
        'sc', Config.sc(i), ...
        'rl', Config.rl(i), ...
        'includeNonEdges', Config.includeNonEdges(i), ...
        'verbose', Config.verbose(i), ...
        'subs', subs_ss, ...
        'nLevels', nLevels ...
        };
    
    tic;
    
    [skeleton pMax pMaxStat] = ...
        org.mensxmachina.bnet.mmskeleton( ...
            data_ss, mmskeleton_params{:} );

    time = toc;
    time
    
    % assert skeleton symmetry
    assertTrue( isequal(tril(skeleton), triu(skeleton)') );
    
    if Config.sc(i)
        
        % assert pMax and pMaxStat symmetry
        assertEqual( tril(pMax), triu(pMax)');
        assertEqual( tril(pMaxStat), triu(pMaxStat)' );
        
        isSkeleton = find(skeleton);
        
        % assert all edges have max p-values <= alpha
        assertTrue( all(pMax(isSkeleton) <= Config.alpha(i)) );
        
        % assert all non-edges have
        % either max p-values > alpha or NaN max p-values
        isNanNonSkeletonPMax = isnan(pMax(~isSkeleton));
        assertTrue( all(pMax(~isSkeleton) > Config.alpha(i) | isNanNonSkeletonPMax) );
        
        if any(isNanNonSkeletonPMax)
            disp('Some NaN max p-values were obtained.');
        end
        
    end
    
    % fill run data
    Run.time(i) = time;
    Run.skeleton{i} = skeleton;
    Run.pMax{i} = pMax;
    Run.pMaxStat{i} = pMaxStat;

end

end

function Run = ceConfigRun(Config)

nConfigs = size(Config, 1);

% create runs dataset
time = zeros(nConfigs, 1);
skeleton = cell(nConfigs, 1);
pMax = cell(nConfigs, 1);
pMaxStat = cell(nConfigs, 1);

Run = dataset(time, skeleton, pMax, pMaxStat);

for i=1:nConfigs % for each configuration

    fprintf('\nRunning configuration %d of %d (%.2f%%)\n', i, nConfigs, i/nConfigs*100);
    
    Config(i, :)
    
    a = [];
    
    % load sample
    load(sprintf('%s_sample_%d', Config.netName{i}, Config.sampleNo(i)), 'a');

    % convert dataset to double
    data = double(a);
    
    % get data up to this sample size
    data = data(1:Config.sampleSize(i), :);

    % convert to Causal Explorer data
    data = data - 1;

    % get number of levels
    nLevels = org.mensxmachina.commons.stats.nlevels(a);

    disp('Applying Causal Explorer''s MMHC...');

    options.threshold = Config.alpha(i);
    options.maxK = Config.nsmax(i);

    [A score stats Run.time(i) Run.skeleton{i}] = ...
        Causal_Explorer('MMHC', data, nLevels, 'MMHC', options, 10, 'BDeu');

end

end

function analyseVarSsRl()  

Run = [];    
    
load('mmskeleton_run_var_ss_rl', 'Run');

netName = unique(Run.netName);
sampleSize = unique(Run.sampleSize);

for i=1:numel(netName) % for each net
    % load net
    load( sprintf('mxm_bnet_%s_net', netName{i}), 'net');
    skeleton.(netName{i}) = net.skeleton;
end

nRuns = size(Run, 1);

% create performance columns
Run.sensitivity = zeros(nRuns, 1);
Run.specificity = zeros(nRuns, 1);

for i=1:nRuns % for each run

    % get performance object
    slp = org.mensxmachina.bnet.skeletonperf( ...
        skeleton.(Run.netName{i}), Run.skeleton{i});
    
    Run.sensitivity(i) = slp.sensitivity;
    Run.specificity(i) = slp.specificity;

end

close all;

% select all performance variables except AUC
PerfVar = TestMmSkeleton.PerfVar;
PerfVar = PerfVar(1:3, :);

for i=1:numel(netName) % for each net
    
    % select runs on this net
    Run_net = Run(strcmp(Run.netName, netName{i}), :);
    
    % assert that for each sample size
    % RLs (0,1) and (2,3) produce the same skeletons
    for j=1:numel(sampleSize)
        skeletons_jss = Run.skeleton(Run_net.sampleSize == sampleSize(j));
        %assertEqual( skeletons_jss{1}, skeletons_jss{2} );
        assertEqual( skeletons_jss{3}, skeletons_jss{4} );
    end
    
    for j=1:size(PerfVar, 1)

        % find RLs giving best performance for each sample size
        
        figure;
        
        [h b] = org.mensxmachina.commons.stats.grpstatplot( ...
            Run_net, 'rl', 'sampleSize', PerfVar.name{j}, 'errorstat', @std);

        % annotate plot
        theylim = ylim;
        ylim([ 0 theylim(2) ]);

        xlabel('Sample size');
        ylabel(sprintf('Mean %s', PerfVar.name{j}));
        title(sprintf('Mean %s learning %s vs sample size', PerfVar.name{j}, netName{i}));

        figureFilename = ...
            sprintf('mmskeleton_%s_mean_%s_vs_ss_foreach_rl', ...
                netName{i}, PerfVar.name{j});

        % print plot
        print(sprintf('-f%d', gcf), '-r300', '-depsc', figureFilename);

        b = grpstats(b, 'sampleSize', PerfVar.rlFun{j}, ...
            'DataVars', sprintf('mean_%s', PerfVar.name{j}), ...
            'VarNames', {'sampleSize', 'GroupCount', 'bestrl'});

        PerfVar.bestrl_ss{j}(i, :) = b.bestrl';
        PerfVar.bestrl_ss{j}(i, :)
        
    end
        
end

for j=1:size(PerfVar, 1)

    fprintf('\nRL leading to %s for each sample size:\n', PerfVar.bestname{j});
    sampleSize'
    mode(PerfVar.bestrl_ss{j})

end
    
end

function analyseCmpWithCeVarSs()  

Run = [];   
    
load('mmskeleton_cmp_with_ce_var_ss', 'Run');

netName = unique(Run.netName);

for i=1:numel(netName) % for each net
    % load net
    load( sprintf('mxm_bnet_%s_net', netName{i}), 'net');
    skeleton.(netName{i}) = net.skeleton;
end

nRuns = size(Run, 1);

% create performance columns
Run.sensitivity = zeros(nRuns, 1);
Run.specificity = zeros(nRuns, 1);

for i=1:nRuns % for each run

    % get performance object
    slp = org.mensxmachina.bnet.skeletonperf( ...
        skeleton.(Run.netName{i}), Run.skeleton{i});
    
    Run.sensitivity(i) = slp.sensitivity;
    Run.specificity(i) = slp.specificity;

end

PerfVar = TestMmSkeleton.PerfVar;
PerfVar = PerfVar(1:3, :); % no AUC
close all;

for i=1:numel(netName) % for each net
    
    % select runs on this net
    Run_net = Run(strcmp(Run.netName, netName{i}), :);
    
    for j=1:size(PerfVar, 1) % for each performance variable
        
        figure;
        
        [h b] = org.mensxmachina.commons.stats.grpstatplot( ...
            Run_net, 'implId', 'sampleSize', PerfVar.name{j}, 'errorstat', @std);

        % annotate plot
        theylim = ylim;
        ylim([ 0 theylim(2) ]);

        xlabel('Sample size');
        ylabel(sprintf('Mean %s', PerfVar.name{j}));
        title(sprintf('Mean %s learning %s vs sample size', PerfVar.label{j}, netName{i}));

        figureFilename = ...
            sprintf('mmskeleton_%s_mean_%s_vs_ss_foreach_impl', ...
                netName{i}, PerfVar.name{j});

        % print plot
        print(sprintf('-f%d', gcf), '-r300', '-depsc', figureFilename);
        
    end
        
end

end

function runAnalyse(runName)
Run = [];   
    
load( sprintf('mmskeleton_%s_run', runName), 'Run');

netName = Run.netName{1};

load( sprintf('mxm_bnet_%s_net', lower(netName)) , 'net');

% create performance columns

% get performance object
slp = org.mensxmachina.bnet.skeletonperf(net.skeleton, Run.skeleton{1});

Run.sensitivity = slp.sensitivity;
Run.specificity = slp.specificity;

fprintf('\nSensitivity = %.2f%%\n', Run.sensitivity*100);
fprintf('\nSpecificity = %.2f%%\n', Run.specificity*100);

% save it with the new columns
save( sprintf('mmskeleton_%s_run', runName), 'Run');

org.mensxmachina.bnet.plotskeletonconfusion(net.skeleton, Run.skeleton{1});

end

function analyseVarSs()  
%ANALYSEVARSS analyse variable sample size runs

Run = [];
    
load('mmskeleton_var_ss', 'Run');

netName = unique(Run.netName);

for i=1:numel(netName) % for each net
    % load net
    load( sprintf('mxm_bnet_%s_net', netName{i}), 'net');
    skeleton.(netName{i}) = net.skeleton;
end

nRuns = size(Run, 1);

% create performance columns

Run.sensitivity = zeros(nRuns, 1);
Run.specificity = zeros(nRuns, 1);
Run.auc = zeros(nRuns, 1);

for i=1:nRuns % for each run

    skeleton_true = skeleton.(Run.netName{i});
    
    % get performance object
    slp = org.mensxmachina.bnet.skeletonperf( ...
        skeleton_true, Run.skeleton{i});
    
    Run.sensitivity(i) = slp.sensitivity;
    Run.specificity(i) = slp.specificity;
    
    Run.auc(i) = org.mensxmachina.bnet.skeletonauc( ...
        skeleton.(Run.netName{i}), Run.pMax{i} );
    
end

% save it with the new columns
save('mmskeleton_var_ss', 'Run');

close all;

sampleSize = unique(Run.sampleSize);
sampleNo = unique(Run.sampleNo);

for i=1:numel(netName) % for each net
        
    pMax_ss = cell(1, numel(sampleSize));
    
    for j=1:numel(sampleSize) % for each sample size
        
        iNetjSs1stRun = Run( ...
            strcmp(Run.netName, netName{i}) & ...
            Run.sampleSize == sampleSize(j) ...
            & Run.sampleNo == sampleNo(1), : );
        
        pMax_ss{j} = iNetjSs1stRun.pMax{1};
        
    end
    
    figure;
    
    [tpr fpr threshold] = org.mensxmachina.bnet.plotskeletonroc( ...
        repmat({skeleton.(netName{i})}, 1, numel(sampleSize)), pMax_ss );
    
    legend( arrayfun(@(i) num2str(i), sampleSize', 'UniformOutput', false) );
    set(gca, 'XTick', [0 iNetjSs1stRun.alpha(1) 1]);
    
    figureFilename = ...
            sprintf('mmskeleton_%s_roc_foreach_ss', netName{i});
    
    % print plot
    print(sprintf('-f%d', gcf), '-r300', '-depsc', figureFilename);    
    
end

PerfVar = TestMmSkeleton.PerfVar;
PerfVar = PerfVar([1 4], :); % time and AUC only

for i=1:numel(netName) % for each net
    
    % select runs on this net
    Run_net = Run(strcmp(Run.netName, netName{i}), :);
    
    for j=1:size(PerfVar, 1) % for each performance variable
        
        figure;
        
        [h b] = org.mensxmachina.commons.stats.grpstatplot( ...
            Run_net, 'sampleSize', PerfVar.name{j}, 'errorstat', @std);

        % annotate plot
        theylim = ylim;
        ylim([ 0 theylim(2) ]);

        xlabel('Sample size');
        ylabel(sprintf('Mean %s', PerfVar.name{j}));
        title(sprintf('Mean %s learning %s vs sample size', PerfVar.label{j}, netName{i}));

        figureFilename = ...
            sprintf('mmskeleton_%s_mean_%s_vs_ss', ...
                netName{i}, PerfVar.name{j});

        % print plot
        print(sprintf('-f%d', gcf), '-r300', '-depsc', figureFilename);
        
    end
        
end
  
end

function analyseCmpWithCe(cmpName)  

Run = [];   
    
load(sprintf('mmskeleton_%s_cmp_with_ce', cmpName), 'Run');

netName = Run.netName{1};

load( sprintf('mxm_bnet_%s_net', lower(netName)), 'net');

nRuns = size(Run, 1);

% create performance columns
Run.sensitivity = zeros(nRuns, 1);
Run.specificity = zeros(nRuns, 1);

for i=1:nRuns % for each run
    
    % get performance object
    slp = org.mensxmachina.bnet.skeletonperf( ...
        net.skeleton, Run.skeleton{i});
    
    Run.sensitivity(i) = slp.sensitivity;
    Run.specificity(i) = slp.specificity;
    
end

% save it with the new columns
save(sprintf('mmskeleton_%s_cmp_with_ce', cmpName), 'Run');

close all;

PerfVar = TestMmSkeleton.PerfVar;
PerfVar = PerfVar(1:3, :); % no AUC

for j=1:size(PerfVar, 1) % for each performance variable

    figure;

    [h b] = org.mensxmachina.commons.stats.grpstatplot( ...
        Run, 'implId', PerfVar.name{j}, 'errorstat', @std);

    % annotate plot
    theylim = ylim;
    ylim([ 0 theylim(2) ]);

    xlabel('Implementation');
    ylabel(sprintf('Mean %s', PerfVar.name{j}));
    title( ...
        sprintf('Mean %s learning %s vs implementation', ...
        PerfVar.label{j}, netName));

    %{
    figureFilename = ...
        sprintf('mmskeleton_%s_mean_%s_vs_impl_monotonic_%d', ...
            netName, PerfVar.name{j});

    % print plot
    print(sprintf('-f%d', gcf), '-r300', '-depsc', figureFilename);
        %}

end
    
end

function PerfVar = PerfVar()
    PerfVar = dataset();
    PerfVar.name = {'time'; 'sensitivity'; 'specificity'; 'auc'};
    PerfVar.bestname = {'minimum execution time'; 'maximum sensitivity'; 'maximum specificity'; 'maximum AUC'};
    PerfVar.rlFun = {@TestMmSkeleton.minrl; @TestMmSkeleton.maxrl; @TestMmSkeleton.maxrl; @TestMmSkeleton.maxrl};
    PerfVar.label = {'time (s)'; 'sensitivity'; 'specificity'; 'AUC'};
    PerfVar.bestrl_ss = cell(4, 1);
end

function minrl = minrl(a)        
    [~, minrl] = min(a);
    minrl = minrl - 1;
end

function maxrl = maxrl(a)        
    [~, maxrl] = max(a);
    maxrl = maxrl - 1;
end
    
end

end