classdef TestNetwork < TestCase
%TESTNETWORK Network class test cases

    properties
        net
        node
    end

    methods

    function obj = TestNetwork(name)
        
        obj = obj@TestCase(name);
        
        % create a weather example network
        obj.net = org.mensxmachina.bnet.weather;
        
        obj.node = struct(...
              'Cloudy', 1, ...
              'Sprinkler', 2, ...
              'Rain', 3, ...
              'WetGrass', 4 );
        
    end
    
    function testsetdescription(obj)
        
        % test assignment of a non-string description
        assertExceptionThrown(@setbaddescription, 'bn:Network:BadDescription');
        
        function setbaddescription()
            obj.net.Description = -i;
        end
        
         % test assignment of a string description
        obj.net.Description = 'Another description';  
        
    end
    
    function testsetclassnames(obj)
        
        % test assignment of an empty matrix
        obj.net.ClassNames = [];
        assertTrue( all(strcmp(obj.net.ClassNames, 'nominal')) );
        
        % test assignment of a cell array of non-strings   
        assertExceptionThrown(@setnonstrcell, 'bn:Network:BadClassNames');
        function setnonstrcell()
            obj.net.ClassNames = repmat(13, 1, 4);
        end     
        
        % test assignment of a cell array of strings of bad length        
        assertExceptionThrown(@setbadlength, 'bn:Network:BadClassNames');
        function setbadlength()
            obj.net.ClassNames = repmat({'nominal'}, 1, 5);
        end     
        
        % test assignment of a cell array of bad strings       
        assertExceptionThrown(@setbadstrings, 'bn:Network:BadClassNames');
        function setbadstrings()
            obj.net.ClassNames = repmat({'blah'}, 1, 4);
        end       
        
        % test assignment of a single classname
        obj.net.ClassNames = 'nominal';
        assertTrue( all(strcmp(obj.net.ClassNames, 'nominal')) );   
        
        % test assignment of a single bad string     
        assertExceptionThrown(@setbadstring, 'bn:Network:BadClassNames');
        function setbadstring()
            obj.net.ClassNames = 'blah';
        end    
        
        % test assignment of a single non-string     
        assertExceptionThrown(@setbadsingle, 'bn:Network:BadClassNames');
        function setbadsingle()
            obj.net.ClassNames = -i;
        end         
        
    end
    
    function testsetvarnames(obj)
        
        % test assignment of a non-cell array of strings   
        assertExceptionThrown(@setnoncellstr, 'bn:Network:BadVarNames');
        function setnoncellstr()
            obj.net.VarNames = -i;
        end
        
        % test assignment of a cell array of non-strings   
        assertExceptionThrown(@setcellnonstr, 'bn:Network:BadVarNames');
        function setcellnonstr()
            obj.net.VarNames = repmat({-i}, 1, 4);
        end     
        
        % test assignment of a cell array of strings of bad length        
        assertExceptionThrown(@setbadlength, 'bn:Network:BadClassNames');
        function setbadlength()
            obj.net.ClassNames = {'var1', 'var2', 'var3', 'var4'};
        end      
        
        % test assignment of a cell array of strings with duplicates        
        assertExceptionThrown(@setduplicates, 'bn:Network:BadClassNames');
        function setduplicates()
            obj.net.ClassNames = {'var1', 'var1', 'var3', 'var4'};
        end 
        
         % test assignment of a valid cell array of strings
        obj.net.ClassNames = repmat({'nominal'}, 1, 4);
        
    end
    
    function testsetlabels(obj)
        
        % test assignment of a non-cell array of strings   
        assertExceptionThrown(@setnoncellstr, 'bn:Network:BadLabels');
        function setnoncellstr()
            obj.net.Labels = -i;
        end
        
        % test assignment of a cell array of non- cell array of strings   
        assertExceptionThrown(@setcellnonstr, 'bn:Network:BadLabels');
        function setcellnonstr()
            obj.net.Labels = repmat({-i}, 1, 4);
        end     
        
        % test assignment of a cell array of cell array of strings
        % of lengths less than 1
        assertExceptionThrown(@setbadlength, 'bn:Network:BadLabels');
        function setbadlength()
            obj.net.Labels = {{}, {'false', 'true'}, {'false', 'true'}, {'false', 'true'}};
        end      
        
        % test assignment of a cell array of cell array of strings
        % with duplicates
        assertExceptionThrown(@setduplicates, 'bn:Network:BadLabels');
        function setduplicates()
            obj.net.Labels = {{'false'}, {'false', 'false'}, {'false', 'true'}, {'false', 'true'}};
        end     
        
        % test valid assignment
        obj.net.Labels = repmat({{'false', 'true'}}, 1, 4);

    end
    
    function testskeleton(obj)
        
        trueskeleton = sparse(4, 4);
        
        trueskeleton(2,1) = 1;
        trueskeleton(3,1) = 1;
        trueskeleton(1,2) = 1;
        trueskeleton(4,2) = 1;
        trueskeleton(1,3) = 1;
        trueskeleton(4,3) = 1;
        trueskeleton(2,4) = 1;
        trueskeleton(3,4) = 1;
        
        assertTrue( isequal(skeleton(obj.net), trueskeleton) );
        
    end
    
    function testparents(obj)
        assertTrue( isempty(parents(obj.net, 1)) );
        assertTrue( isequal(parents(obj.net, 2), 1) );
        assertTrue( isequal(parents(obj.net, 3), 1) );
        assertTrue( isequal(parents(obj.net, 4), [2 3]) );
    end
    
    function testnlevels(obj)
        assertTrue(all(arrayfun(@(i) nlevels(obj.net, i) == 2, 1:4)) );
    end
    
    function testrnd(obj)
        a = rnd(obj.net, 10);
    end
    
    %{
    function testsetcpd(obj)
        
        assertExceptionThrown(@setnoncellarraycpd, 'bn:Network:BadCPD');
        assertExceptionThrown(@setcpdofbadlength, 'bn:Network:BadCPD');
        
        function setnoncellarraycpd()
            obj.net.CPD = -i;
        end
        
        function setcpdofbadlength()
            obj.net.CPD = cell(1, 5);
        end
        
    end
    %}
    end
  
end 
